<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;


/**
 * Auth Controller Class
 */
class AuthController extends Controller
{
    /**
     * @unauthenticated
     * Login user to get sanctum token
     * 
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function login(Request $request)
    {
        $request->validate([
            'email' => 'required|email',
            'password' => 'required',
        ]);
        $credentials = $request->only('email', 'password');

        if (auth()->attempt($credentials)) {
            $token = $request->user()->createToken('authToken')->plainTextToken;

            return response()->json(['token' => $token, 'type' => 'Bearer'], 200);
        }
        abort(401, 'Bad credentials.');
    }
}
