<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Models\ComponentType;
use Illuminate\Http\Request;

class ComponentTypeController extends Controller
{
    /**
     * @unauthenticated
     * List all components records and return a collection of components
     * 
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function list(Request $request) {
        $request->validate([
            'per_page' => 'nullable|numeric',
            'page' => 'nullable|numeric'
        ]);

        $perPage = $request->per_page ?? 20;
        return response()->json(ComponentType::paginate($perPage));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function store(Request $request) {
        $request->validate([
            'name' => 'required'
        ]);

        $component = ComponentType::create($request->all());
        return response()->json($component, 201);

    }


    /**
     * Display the specified resource.
     *
     * @param Request $request
     * @param string $id
     * @return \Illuminate\Http\JsonResponse
     */
    public function show(Request $request, $id) {
        $component = ComponentType::findOrFail($id);
        return response()->json($component);
    }


    /**
     * Update the specified resource in storage.
     *
     * @param Request $request
     * @param string $id
     * @return \Illuminate\Http\JsonResponse
     */
    public function update(Request $request, $id) {
        $request->validate([
            'name' => 'required'
        ]);

        $component = ComponentType::findOrFail($id);
        $component->update($request->all());
        return response()->json($component);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param string $id
     * @return \Illuminate\Http\JsonResponse
     */

    public function delete($id) {
        $component = ComponentType::findOrFail($id);
        $component->delete();
        return response()->json(['message' => 'Component deleted successfully']);

    }
}
