<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

use PHPMailer\PHPMailer\PHPMailer;


class EmailController extends Controller
{
    public array $defaultEmail;

    public function __construct()
    {
        $this->defaultEmail = [
            'name' => env('MAIL_FROM_NAME', ''),
            'email'=> env('MAIL_FROM_ADDRESS',''),
        ];
    }

    /**
     * Send email to recipient
     * 
     * @unauthenticated
     */
    public function send(Request $request)
    {
        $request->validate([
            'name' => 'required|string',
            'email' => 'required|email',
            'phone' => 'string',
            'company' => 'required|string',
            
            'address' => 'required|string',
            'city' => 'required|string',
            'postalCode' => 'required|string',
            'country' => 'required|string',

            'subject' => 'required|string',
            'message' => 'required|string',
        ]);
        $data = $request->all();

        $result = $this->sendMail(
            (object) [
                'name' => $data['name'],
                'email' => $data['email']
            ],
            (object) [
                'name' => $this->defaultEmail['name'],
                'email' => env('CONTACT_EMAIL', '') 
            ],
            $data['subject'],
            view('mails.contact', $data)->render()
        );

        return $result === true ? response()->json(['message' => 'Email sent successfully']) : $result;
    }

    /**
     * Send email to recipient
     * 
     * @param object $sender (name, email)
     * @param object $recipient (name, email)
     * @param string $subject
     * @param string $content (HTML content)
     * @return boolean true if email is sent, false otherwise
     */
    private function sendMail($sender, $recipient, $subject, $content)
    {
        $mail = new PHPMailer(true);

        try {
            /*$mail->isSMTP();
            $mail->Host = 'dev.cryopal.com';
            $mail->SMTPAuth = true;
            $mail->Username = env('MAIL_FROM_ADDRESS', '');
            $mail->Password = env('MAIL_PASSWORD', '');
            $mail->SMTPSecure = PHPMailer::ENCRYPTION_SMTPS;
            $mail->Port = 465;

            // Recipients
            $mail->setFrom($sender->email, $sender->name);
            $mail->addAddress($recipient->email, $recipient->name);

            $mail->isHTML(true);
            $mail->CharSet = 'UTF-8';
            $mail->Subject = $subject;
            $mail->Body = $content;

            $mail->send();*/
            
            $mail->isSMTP();
            $mail->Host = 'ssl0.ovh.net';
            $mail->SMTPAuth = true;
            $mail->Username = 'infos@toulouseweb.com';
            $mail->Password = '4s8uSW7x';
            $mail->SMTPSecure = PHPMailer::ENCRYPTION_SMTPS;
            $mail->Port = 587;

            // Recipients
            $mail->setFrom('infos@toulouseweb.com', 'infos@toulouseweb.com');
            $mail->addAddress('rado.amws@gmail.com', 'rado.amws@gmail.com');

            $mail->isHTML(true);
            $mail->CharSet = 'UTF-8';
            $mail->Subject = $subject;
            $mail->Body = $content;

            $mail->send();
            
            return true;
        } catch (\Exception $e) {
            echo $mail->ErrorInfo;
            return response()->json(['message'=> $e->getMessage()], 400);
        }
    }
}
