<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Resources\GroupResource;
use Illuminate\Http\Request;
use App\Models\Group;


class GroupController extends Controller
{
    /**
     * List all groups records and return a collection of groups
     * 
     * @unauthenticated
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function list(Request $request) {
        $request->validate([
            'per_page' => 'nullable|numeric',
            'page' => 'nullable|numeric'
        ]);

        $perPage = $request->per_page ?? 10;
        return response()->json(GroupResource::collection(Group::paginate($perPage)));
    }

    /**
     * Store a newly created group record
     * 
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function store(Request $request) {
        $request->validate([
            'name' => 'required|string',
            'slug' => 'required|string',
        ]);

        $group = Group::create($request->all());
        return response()->json(GroupResource::make($group), 201);
    }

    /**
     * Get data of a group record from its id
     * 
     * @unauthenticated
     * @param Request $request
     * @param string $id (uuid format)
     * @return \Illuminate\Http\JsonResponse
     */
    public function show(Request $request, $id) {
        $group = Group::findOrFail($id);
        return response()->json(GroupResource::make($group));
    }

    /**
     * Update a group record
     * 
     * @param Request $request
     * @param string $id (uuid format)
     * @return \Illuminate\Http\JsonResponse
     */
    public function update(Request $request, $id) {
        $request->validate([
            'name' => 'required|string',
            'slug' => 'required|string',
        ]);

        $group = Group::findOrFail($id);
        $group->update($request->all());
        return response()->json(GroupResource::make($group));
    }

    /**
     * Delete a group record
     * 
     * @param Request $request
     * @param string $id (uuid format)
     * @return \Illuminate\Http\JsonResponse
     */
    public function delete(Request $request, $id) {
        $group = Group::findOrFail($id);
        $group->delete();
        return response()->json(null, 204);
    }
}
