<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Models\Parameter;
use Illuminate\Http\Request;

class ParameterController extends Controller
{
    /**
     * @unauthenticated
     * List all component's parameters records and return a collection of parameters
     * 
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function list(Request $request) {
        $request->validate([
            'per_page' => 'nullable|numeric',
            'page' => 'nullable|numeric'
        ]);

        $perPage = $request->per_page ?? 20;
        return response()->json(Parameter::paginate($perPage));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function store(Request $request) {
        $request->validate([
            'name' => 'required|string',
            'value' => 'required|string',
            'component_id' => 'required|string|exists:components,id',
            'file' => 'nullable|file'
        ]);

        // if parameter is a file, store it in the storage
        if ($request->hasFile('file')) {
            $file = $request->file('file');
            $path = $file->store('parameter_uploads', 'public');
            $request->merge(['value' => $path]);
        }

        $component = Parameter::create($request->all());
        return response()->json($component, 201);
    }

    /**
     * Display the specified resource.
     *
     * @param Request $request
     * @param string $id
     * @return \Illuminate\Http\JsonResponse
     */
    public function show(Request $request, $id) {
        $component = Parameter::findOrFail($id);
        return response()->json($component);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param Request $request
     * @param string $id
     * @return \Illuminate\Http\JsonResponse
     */
    public function update(Request $request, $id) {
        $component = Parameter::findOrFail($id);
        $request->validate([
            'name' => 'required|string',
            'value' => 'required|string',
            'component_id' => 'required|string|exists:components,id',
        ]);

        $component->update($request->all());
        return response()->json($component);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param Request $request
     * @param string $id
     * @return \Illuminate\Http\JsonResponse
     */

    public function delete(Request $request, $id) {
        $component = Parameter::findOrFail($id);
        $component->delete();
        return response()->json(['message' => 'Component deleted successfully']);

    }
}
