<?php

use App\Http\Controllers\Api\AuthController;
use App\Http\Controllers\Api\ComponentController;
use App\Http\Controllers\Api\ComponentTypeController;
use App\Http\Controllers\Api\EmailController;
use App\Http\Controllers\Api\GroupController;
use App\Http\Controllers\Api\MenuController;
use App\Http\Controllers\Api\PageController;
use App\Http\Controllers\Api\ParameterController;
use Illuminate\Support\Facades\Route;
use App\Http\Controllers\Api\MediaController;


Route::prefix('v1')->group(function () {

    /* 
    * Public routes
    * All routes in this group are public and do not require authentication
    */
    Route::controller(AuthController::class)->prefix('auth')->group(function () {
        Route::post('/login', 'login')->name('auth.login');
    });

    Route::controller(MenuController::class)->prefix('menu')->group(function () {
        Route::get('/', 'list')->name('menu.list');
        Route::get('/{id}', 'show')->name('menu.show');
    });

    Route::controller(PageController::class)->prefix('page')->group(function () {
        Route::get('/', 'list')->name('page.list');
        Route::get('/{id}', 'show')->name('page.show');
    });

    Route::controller(GroupController::class)->prefix('group')->group(function () {
        Route::get('/', 'list')->name('group.list');
        Route::get('/{id}', 'show')->name('group.show');
    });

    Route::controller(ComponentController::class)->prefix('component')->group(function () {
        Route::get('/', 'list')->name('component.list');
        Route::get('/{id}', 'show')->name('component.show');
    });

    Route::controller(ComponentTypeController::class)->prefix('component-type')->group(function () {
        Route::get('/', 'list')->name('component-type.list');
        Route::get('/{id}', 'show')->name('component-type.show');
    });

    Route::controller(ParameterController::class)->prefix('parameter')->group(function () {
        Route::get('/', 'list')->name('parameter.list');
        Route::get('/{id}', 'show')->name('parameter.show');
    });

    // Serve file uploaded as parameter
    Route::controller(MediaController::class)->prefix('media')->group(function () {
        Route::get('/{folder}/{filename}', 'show')->name('media.show');
    });

    // Send email to from user to contact
    Route::controller(EmailController::class)->prefix('email')->group(function () {
        Route::post('/', 'send')->name('email.send');
    });

    /* 
    * Authenticated routes (protected routes)
    * All routes in this group are protected by Sanctum middleware
    */
    Route::controller(MenuController::class)->prefix('menu')->middleware('auth:sanctum')->group(function () {
        Route::post('/', 'store')->name('menu.store');
        Route::put('/{id}', 'update')->name('menu.update');
        Route::delete('/{id}', 'delete')->name('menu.delete');
    });

    Route::controller(PageController::class)->prefix('page')->middleware('auth:sanctum')->group(function () {
        Route::post('/', 'store')->name('page.store');
        Route::put('/{id}', 'update')->name('page.update');
        Route::delete('/{id}', 'delete')->name('page.delete');
    });

    Route::controller(GroupController::class)->prefix('group')->middleware('auth:sanctum')->group(function () {
        Route::post('/', 'store')->name('group.store');
        Route::put('/{id}', 'update')->name('group.update');
        Route::delete('/{id}', 'delete')->name('group.delete');
    });

    Route::controller(ComponentController::class)->prefix('component')->middleware('auth:sanctum')->group(function () {
        Route::post('/', 'store')->name('component.store');
        Route::put('/{id}', 'update')->name('component.update');
        Route::delete('/{id}', 'delete')->name('component.delete');
    });

    Route::controller(ComponentTypeController::class)->prefix('component-type')->middleware('auth:sanctum')->group(function () {
        Route::post('/', 'store')->name('component-type.store');
        Route::put('/{id}', 'update')->name('component-type.update');
        Route::delete('/{id}', 'delete')->name('component-type.delete');
    });

    Route::controller(ParameterController::class)->prefix('parameter')->middleware('auth:sanctum')->group(function () {
        Route::post('/', 'store')->name('parameter.store');
        Route::put('/{id}', 'update')->name('parameter.update');
        Route::delete('/{id}', 'delete')->name('parameter.delete');
    });
});
